@@ -45,6 +45,12 @@ tpm = function(param, byrow = FALSE) {
   if(byrow) Gamma = t(Gamma)
   
   Gamma = Gamma / rowSums(Gamma)
+  
+  # naming
+  statenames <- paste0("S", 1:N)
+  rownames(Gamma) <- statenames
+  colnames(Gamma) <- statenames
+  
   Gamma
 }
 
@@ -74,6 +80,7 @@ tpm = function(param, byrow = FALSE) {
 #' @return array of transition probability matrices of dimension c(N,N,n)
 #' @export
 #' @import RTMB
+#' @importFrom stats na.omit
 #'
 #' @examples
 #' Z = matrix(runif(200), ncol = 2)
@@ -95,6 +102,16 @@ tpm_g = function(Z, beta, byrow = FALSE, ad = NULL, report = TRUE){
   
   # report quantities for easy use later
   if(report) {
+    #if(is.null(colnames(beta))){
+      # Setting colnames for beta: Inherit colnames from Z
+    colnames(beta) <- colnames(Z)
+    #}
+    if(is.null(rownames(beta))){
+      # Setting rownames: depends on byrow
+      names <- outer(paste0("S", 1:N, ">"), paste0("S", 1:N), FUN = paste0) # matrix
+      diag(names) <- NA
+      rownames(beta) <- na.omit(if (byrow) c(t(names)) else c(names))
+    }
     RTMB::REPORT(beta)
   }
   
@@ -148,6 +165,11 @@ tpm_g = function(Z, beta, byrow = FALSE, ad = NULL, report = TRUE){
     }
   }
   
+  # naming
+  statenames <- paste0("S", 1:N)
+  rownames(Gamma) <- statenames
+  colnames(Gamma) <- statenames
+  
   Gamma
 }
 
@@ -327,6 +349,13 @@ tpm_cont = function(Q, timediff, ad = NULL, report = TRUE){
       Qube[,,t] = as.matrix(Matrix::expm(Q * timediff[t])) # Matrix::expm for AD
     }
   }
+  
+  # naming
+  N <- nrow(Q)
+  statenames <- paste0("S", 1:N)
+  rownames(Qube) <- statenames
+  colnames(Qube) <- statenames
+  
   Qube
 }
 
@@ -370,6 +399,11 @@ generator = function(param, byrow = FALSE, report = TRUE) {
     RTMB::REPORT(Q)
   }
   
+  # naming
+  statenames <- paste0("S", 1:N)
+  rownames(Q) <- statenames
+  colnames(Q) <- statenames
+  
   Q
 }
 
@@ -429,6 +463,12 @@ tpm_emb = function(param = NULL){
     omega = t(omega) / apply(omega, 2, sum)
   }
   
+  # naming
+  N = nrow(omega)
+  statenames <- paste0("S", 1:N)
+  rownames(omega) <- statenames
+  colnames(omega) <- statenames
+  
   omega
 }
 
@@ -506,6 +546,11 @@ tpm_emb_g = function(Z, beta, report = TRUE){
     omega[,,t] = t(O) / apply(O, 2, sum)
   }
   
+  # naming
+  statenames <- paste0("S", 1:N)
+  rownames(omega) <- statenames
+  colnames(omega) <- statenames
+  
   omega
 }
 
